"use strict";

var _jestNative = require("@testing-library/jest-native");
var _react = _interopRequireDefault(require("react"));
var _testUtils = require("../../test-utils");
var _hooks = require("./hooks");
var _Toast = _interopRequireDefault(require("./Toast"));
var _theme = _interopRequireDefault(require("../../theme"));
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// @ts-ignore:next-line

jest.mock('react-native-safe-area-context', () => {
  const inset = {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0
  };
  return {
    SafeAreaProvider: jest.fn().mockImplementation(({
      children
    }) => children),
    SafeAreaConsumer: jest.fn().mockImplementation(({
      children
    }) => children(inset)),
    useSafeAreaInsets: jest.fn().mockImplementation(() => inset)
  };
});
jest.mock('./hooks', () => {
  return {
    useKeyboard: jest.fn().mockImplementation(() => ({
      keyboardHeight: 200,
      isKeyboardVisible: true
    }))
  };
});
describe('Toast', () => {
  expect.extend({
    toHaveStyle: _jestNative.toHaveStyle
  });
  beforeEach(() => {
    jest.clearAllMocks();
    jest.clearAllTimers();
    jest.restoreAllMocks();
    (0, _testUtils.cleanup)();
  });
  test('should print warn when no root component', () => {
    // given
    const warnSpy = jest.spyOn(console, 'warn');

    // when
    _Toast.default.show({
      text: 'lorem ipsum dolor'
    });

    // then
    expect(warnSpy).toBeCalledWith('Please add <Toast/> to your root view');
  });
  test('should render correctly', () => {
    // when
    const {
      toJSON,
      queryByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {}));

    // then
    expect(toJSON()).toMatchSnapshot();
    expect(queryByTestId('toast-wrapper')).toBeNull();
  });
  test('should show toast', () => {
    // when
    const {
      getByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {}));
    _Toast.default.show({
      text: 'lorem ipsum dolor'
    });
    const toastContainer = getByTestId('toast-container');

    // then
    expect(toastContainer).toBeTruthy();
  });
  test('should show toast when keyboard visible', () => {
    // given
    // @ts-ignore
    _hooks.useKeyboard.mockImplementation(() => ({
      keyboardHeight: 150,
      isKeyboardVisible: true
    }));

    // when
    const {
      getByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {
      extraPaddingBottom: 10
    }));
    _Toast.default.show({
      text: 'lorem ipsum dolor'
    });
    const toastWrapper = getByTestId('toast-wrapper');

    // then
    expect(toastWrapper).toBeTruthy();
    expect(toastWrapper).toHaveStyle({
      paddingBottom: _theme.default.spacing['2xs'] + 150 + 10
    });
  });
  test('should render correct bottom offset with offset 40', () => {
    const dataWithBottomOffSet = {
      variant: 'warning',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam',
      bottomOffset: 40
    };

    // @ts-ignore
    _hooks.useKeyboard.mockImplementation(() => ({
      keyboardHeight: 150,
      isKeyboardVisible: true
    }));

    // when
    const {
      getByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {
      ignoreKeyboard: true,
      extraPaddingBottom: 10
    }));
    _Toast.default.show(dataWithBottomOffSet);
    const toastWrapper = getByTestId('toast-wrapper');

    // then
    expect(toastWrapper).toBeTruthy();
    expect(toastWrapper).toHaveStyle({
      paddingBottom: 10 + 40
    });
  });
  test('should hide toast', () => {
    jest.useFakeTimers();
    // when
    const {
      queryByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {}));
    _Toast.default.show({
      text: 'lorem ipsum dolor'
    });
    jest.runAllTimers();
    const toastContainer = queryByTestId('toast-container');

    // then
    expect(toastContainer).toBeNull();
  });
  test('ignore keyboard', () => {
    // given
    // @ts-ignore
    _hooks.useKeyboard.mockImplementation(() => ({
      keyboardHeight: 150,
      isKeyboardVisible: true
    }));

    // when
    const {
      getByTestId
    } = (0, _testUtils.render)(/*#__PURE__*/(0, _jsxRuntime.jsx)(_Toast.default, {
      ignoreKeyboard: true,
      extraPaddingBottom: 10
    }));
    _Toast.default.show({
      text: 'lorem ipsum dolor'
    });
    const toastWrapper = getByTestId('toast-wrapper');

    // then
    expect(toastWrapper).toBeTruthy();
    expect(toastWrapper).toHaveStyle({
      paddingBottom: 10
    });
  });
});
//# sourceMappingURL=Toast.test.js.map