"use strict";

import React, { forwardRef, useEffect } from 'react';
import { Easing, TextInput, I18nManager } from 'react-native';
import theme from '../../theme';
import Box from '../Box/Box';
import { InputIcon } from './InputIcon';
import { InputLabel } from './InputLabel';
import { InputHelpText } from './InputHelpText';
import { useInputRef, useInputValue, useOutlineLabelVisibility, useTogglePasswordVisibility } from './hooks';
import { getBorderColor, getPlaceholderText } from './utils';
import { backgroundColor, border, createRestyleComponent, createVariant, layout, position, spacing } from '@ergenekonyigit/restyle';
import Text from '../Text/Text';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const sizeVariant = createVariant({
  property: 'size',
  themeKey: 'inputSizeVariants'
});
const BaseInput = createRestyleComponent([layout, spacing, border, backgroundColor, position, sizeVariant], TextInput);
const isRTL = I18nManager.isRTL;
const Input = /*#__PURE__*/forwardRef(({
  size = 'large',
  label,
  subLabel = '',
  labelFixed,
  placeholder,
  helpText,
  errorText,
  successText,
  icon,
  success = false,
  error = false,
  secureTextEntry = false,
  required = false,
  disabled = false,
  editable = true,
  easing = Easing.inOut(Easing.ease),
  testID = 'input',
  leadingText,
  trailingText,
  ...rest
}, ref) => {
  const [focused, setFocused] = React.useState(false);
  const [errorState, setErrorState] = React.useState(false);
  const [successState, setSuccessState] = React.useState(false);
  const [variantIconName, setVariantIconName] = React.useState(null);
  const innerRef = useInputRef();
  const {
    value,
    isControlled,
    setUncontrolledValue
  } = useInputValue({
    value: rest.value,
    defaultValue: rest.defaultValue
  });
  const placeholderText = getPlaceholderText({
    label,
    subLabel,
    labelFixed,
    placeholder,
    value,
    focused
  });
  const borderColor = getBorderColor({
    focused,
    successState,
    errorState
  });
  const inputHeight = {
    small: theme.inputSizeVariants.small.height,
    medium: theme.inputSizeVariants.medium.height,
    large: theme.inputSizeVariants.large.height
  }[size];
  const {
    passwordVisibility,
    setPasswordVisibility,
    passwordVisibilityIcon,
    handlePasswordVisibility
  } = useTogglePasswordVisibility(secureTextEntry);
  useEffect(() => {
    setPasswordVisibility(secureTextEntry);
  }, [secureTextEntry, setPasswordVisibility]);
  const {
    startAnimation,
    stopAnimation,
    animatedViewProps,
    animatedTextProps
  } = useOutlineLabelVisibility({
    theme,
    easing,
    inputHeight,
    focused,
    value,
    disabled,
    helpText,
    errorText,
    successText,
    errorState,
    successState
  });
  useEffect(() => {
    setErrorState(error);
    setSuccessState(success);
    if (icon) {
      setVariantIconName(null);
    } else if (error) {
      setVariantIconName('close-fill');
    } else if (success) {
      setVariantIconName('check-fill');
    }
  }, [error, success, icon]);
  const handleFocus = async e => {
    if (disabled || !editable) {
      return;
    }
    await startAnimation();
    setFocused(true);
    setVariantIconName(null);
    setErrorState(false);
    setSuccessState(false);
    rest.onFocus?.(e);
  };
  const handleBlur = async e => {
    if (disabled || !editable) {
      return;
    }
    if (!value) {
      await stopAnimation();
    }
    setFocused(false);
    rest.onBlur?.(e);
  };
  const handleChangeText = nextValue => {
    if (disabled || !editable) {
      return;
    }
    if (!isControlled) {
      setUncontrolledValue(nextValue);
    }
    rest.onChangeText?.(nextValue);
  };
  React.useImperativeHandle(ref, () => ({
    focus: () => innerRef.current?.focus(),
    clear: () => innerRef.current?.clear(),
    setNativeProps: args => innerRef.current?.setNativeProps(args),
    isFocused: () => innerRef.current?.isFocused() || false,
    blur: () => innerRef.current?.blur()
  }));
  return /*#__PURE__*/_jsxs(Box, {
    py: '2xs',
    children: [/*#__PURE__*/_jsx(InputLabel, {
      label: label,
      subLabel: ` ${subLabel} `,
      labelFixed: labelFixed,
      errorState: errorState,
      successState: successState,
      animatedViewProps: animatedViewProps,
      animatedTextProps: animatedTextProps,
      inputHeight: inputHeight,
      required: required
    }), /*#__PURE__*/_jsxs(Box, {
      flexDirection: "row",
      alignItems: "center",
      borderWidth: 1,
      borderRadius: 'm',
      borderColor: borderColor,
      backgroundColor: disabled ? 'neutralLightest' : 'neutralFull',
      px: "m",
      zIndex: "layer_0",
      accessibilityLabel: `${testID}-box`,
      testID: `${testID}-box`,
      children: [leadingText && !disabled ? /*#__PURE__*/_jsx(Box, {
        mr: "2xs",
        children: /*#__PURE__*/_jsx(Text, {
          color: "neutralDark",
          variant: "subtitle2Medium",
          accessibilityLabel: "input-leading-text",
          children: leadingText
        })
      }) : null, /*#__PURE__*/_jsx(BaseInput, {
        ...rest,
        flex: 1,
        ref: innerRef,
        multiline: false,
        height: inputHeight,
        bg: "transparent",
        onFocus: handleFocus,
        onBlur: handleBlur,
        placeholder: placeholderText,
        placeholderTextColor: theme.colors.neutralLight,
        secureTextEntry: passwordVisibility,
        onChangeText: handleChangeText,
        value: value,
        disabled: disabled,
        editable: !disabled,
        accessibilityLabel: testID,
        testID: testID,
        textAlign: isRTL ? 'right' : 'left'
      }), trailingText ? /*#__PURE__*/_jsx(Box, {
        ml: "m",
        mr: icon ? 'm' : 'none',
        children: /*#__PURE__*/_jsx(Text, {
          color: "neutralDark",
          variant: "subtitle2Medium",
          accessibilityLabel: "input-trailing-text",
          children: trailingText
        })
      }) : null, /*#__PURE__*/_jsx(InputIcon, {
        secureTextEntry: secureTextEntry,
        icon: icon,
        variantIconName: variantIconName,
        focused: focused,
        successState: successState,
        errorState: errorState,
        passwordVisibilityIcon: passwordVisibilityIcon,
        handlePasswordVisibility: handlePasswordVisibility
      })]
    }), /*#__PURE__*/_jsx(InputHelpText, {
      helpText: helpText,
      errorState: errorState,
      errorText: errorText,
      successState: successState,
      successText: successText
    })]
  });
});
export default Input;
//# sourceMappingURL=Input.js.map