"use strict";

import React, { forwardRef, useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { Easing, Pressable, TouchableOpacity, StyleSheet } from 'react-native';
import theme from '../../theme';
import Box from '../Box/Box';
import Icon from '../Icon/Icon';
import Text from '../Text/Text';
import SelectBottomSheet from '../SelectBottomSheet/SelectBottomSheet';
import { InputLabel } from '../Input/InputLabel';
import { InputHelpText } from '../Input/InputHelpText';
import { useOutlineLabelVisibility } from '../Input/hooks';
import { getBorderColor } from '../Input/utils';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const useSelectState = selectedOptions => {
  const [selectedValues, setSelectedValues] = useState(Array.isArray(selectedOptions) ? selectedOptions : [selectedOptions]);
  const bottomSheetSelectionRef = useRef([]);
  const resetSelections = useCallback(() => {
    setSelectedValues([]);
  }, []);
  const updateSelections = useCallback(newSelections => {
    setSelectedValues(newSelections);
  }, []);
  return {
    selectedValues,
    updateSelections,
    resetSelections,
    bottomSheetSelectionRef
  };
};
const useDisplayValue = props => {
  const {
    multiple,
    selectedOptionObjects,
    selectedValues,
    maxDisplayItems = 2
  } = props;
  const getDisplayValues = useCallback(() => {
    if (selectedValues.length === 0) {
      return {
        visibleText: '',
        remainingCount: 0
      };
    }
    if (multiple) {
      const selectedLabels = selectedOptionObjects.map(option => option.label);
      if (selectedLabels.length <= maxDisplayItems) {
        return {
          visibleText: selectedLabels.join(', '),
          remainingCount: 0
        };
      } else {
        const visibleLabels = selectedLabels.slice(0, maxDisplayItems);
        const remainingCount = selectedLabels.length - maxDisplayItems;
        return {
          visibleText: visibleLabels.join(', '),
          remainingCount
        };
      }
    }
    return {
      visibleText: selectedOptionObjects.length > 0 ? selectedOptionObjects[0].label : '',
      remainingCount: 0
    };
  }, [selectedOptionObjects, multiple, selectedValues, maxDisplayItems]);
  return {
    getDisplayValues
  };
};
const useSortedOptions = props => {
  const {
    options,
    selectedValues,
    isBottomSheetVisible,
    wasCleared
  } = props;
  const originalOptionsRef = useRef([]);
  useEffect(() => {
    if (originalOptionsRef.current.length === 0) {
      originalOptionsRef.current = [...options];
    }
  }, [options]);
  return useMemo(() => {
    if (wasCleared || !isBottomSheetVisible) {
      return options;
    }
    if (selectedValues.length === 0) {
      return options;
    }
    const selectedOptions = options.filter(option => selectedValues.includes(option.id));
    const unselectedOptions = options.filter(option => !selectedValues.includes(option.id));
    return [...selectedOptions, ...unselectedOptions];
  }, [options, selectedValues, isBottomSheetVisible, wasCleared]);
};
const Select = /*#__PURE__*/forwardRef(({
  label,
  subLabel,
  labelFixed = false,
  placeholder = '',
  helpText,
  error = false,
  errorText,
  success = false,
  successText,
  required = false,
  disabled = false,
  size = 'medium',
  options = [],
  selectedOptions = [],
  testID = 'select',
  onSelect,
  bottomSheetTitle,
  closeButtonLabel = '',
  selectButtonLabel,
  multiple = true,
  maxVisibleItems = 6,
  ...rest
}, ref) => {
  const [focused, setFocused] = useState(false);
  const [errorState, setErrorState] = useState(error);
  const [successState, setSuccessState] = useState(success);
  const [isBottomSheetVisible, setIsBottomSheetVisible] = useState(false);
  const [wasCleared, setWasCleared] = useState(false);
  const {
    selectedValues,
    updateSelections,
    resetSelections,
    bottomSheetSelectionRef
  } = useSelectState(selectedOptions);
  const selectedOptionObjects = useMemo(() => options.filter(option => selectedValues.includes(option.id)), [options, selectedValues]);
  const sortedOptions = useSortedOptions({
    options,
    selectedValues,
    multiple,
    isBottomSheetVisible,
    wasCleared
  });
  const inputHeight = theme.inputSizeVariants[size].height;
  const borderColor = useMemo(() => getBorderColor({
    focused,
    successState,
    errorState
  }), [focused, successState, errorState]);
  const {
    getDisplayValues
  } = useDisplayValue({
    multiple,
    selectedOptionObjects,
    selectedValues,
    maxDisplayItems: 2
  });
  const {
    startAnimation,
    animatedViewProps,
    animatedTextProps
  } = useOutlineLabelVisibility({
    theme,
    easing: Easing.inOut(Easing.ease),
    inputHeight,
    focused,
    value: getDisplayValues().visibleText || '',
    disabled,
    helpText,
    errorText,
    successText,
    errorState,
    successState
  });
  useEffect(() => {
    setErrorState(error);
    setSuccessState(success);
  }, [error, success]);
  const openBottomSheet = useCallback(async () => {
    if (disabled) {
      return;
    }
    await startAnimation();
    setFocused(true);
    bottomSheetSelectionRef.current = [...selectedValues];
    setWasCleared(false);
    setIsBottomSheetVisible(true);
  }, [disabled, startAnimation, selectedValues, bottomSheetSelectionRef]);
  const closeBottomSheet = useCallback(() => {
    setIsBottomSheetVisible(false);
    setFocused(false);
  }, []);
  const confirmSelection = useCallback(() => {
    const newSelections = [...bottomSheetSelectionRef.current];
    updateSelections(newSelections);
    if (onSelect) {
      onSelect(newSelections);
    }
    closeBottomSheet();
  }, [onSelect, closeBottomSheet, bottomSheetSelectionRef, updateSelections]);
  const handleClear = useCallback(event => {
    event?.stopPropagation();
    resetSelections();
    if (onSelect) {
      onSelect([]);
    }
    setWasCleared(true);
  }, [onSelect, resetSelections]);
  const handleBottomSheetSelection = useCallback(selectedIds => {
    bottomSheetSelectionRef.current = selectedIds;
  }, [bottomSheetSelectionRef]);
  const {
    visibleText,
    remainingCount
  } = useMemo(() => getDisplayValues(), [getDisplayValues]);
  const hasSelectedValue = useMemo(() => selectedValues.length > 0, [selectedValues]);
  const iconColor = useMemo(() => {
    if (disabled) {
      return 'neutralLight';
    } else if (focused || hasSelectedValue) {
      return 'neutralDarker';
    } else {
      return 'neutralLight';
    }
  }, [focused, hasSelectedValue, disabled]);
  React.useImperativeHandle(ref, () => ({
    focus: openBottomSheet,
    blur: closeBottomSheet,
    clear: () => {
      handleClear();
    }
  }));
  return /*#__PURE__*/_jsxs(Box, {
    py: "2xs",
    ...rest,
    children: [/*#__PURE__*/_jsx(InputLabel, {
      label: label,
      subLabel: subLabel,
      labelFixed: labelFixed,
      errorState: errorState,
      successState: successState,
      animatedViewProps: animatedViewProps,
      animatedTextProps: animatedTextProps,
      inputHeight: inputHeight,
      required: required
    }), /*#__PURE__*/_jsx(Pressable, {
      onPress: openBottomSheet,
      disabled: disabled,
      testID: `${testID}-pressable`,
      children: /*#__PURE__*/_jsxs(Box, {
        flexDirection: "row",
        alignItems: "center",
        borderWidth: 1,
        borderRadius: 'm',
        borderColor: borderColor,
        backgroundColor: disabled ? 'neutralLightest' : 'neutralFull',
        px: "m",
        height: inputHeight,
        zIndex: "layer_0",
        accessibilityLabel: `${testID}-box`,
        testID: `${testID}-box`,
        children: [/*#__PURE__*/_jsxs(Box, {
          flex: 1,
          flexDirection: "row",
          alignItems: "center",
          justifyContent: "space-between",
          testID: `${testID}-text-container`,
          children: [/*#__PURE__*/_jsx(Text, {
            variant: "body2",
            color: visibleText ? 'neutralDarker' : 'neutralLight',
            testID: `${testID}-value`,
            numberOfLines: 1,
            ellipsizeMode: "tail",
            style: {
              flex: 1
            },
            children: visibleText || placeholder
          }), remainingCount > 0 && /*#__PURE__*/_jsxs(Text, {
            variant: "body2",
            color: "neutralDarker",
            testID: `${testID}-remaining-count`,
            marginRight: "m",
            children: ["+", remainingCount]
          })]
        }), /*#__PURE__*/_jsxs(Box, {
          flexDirection: "row",
          alignItems: "center",
          children: [hasSelectedValue && !disabled && /*#__PURE__*/_jsx(TouchableOpacity, {
            onPress: handleClear,
            hitSlop: {
              top: 10,
              right: 10,
              bottom: 10,
              left: 10
            },
            testID: `${testID}-clear`,
            style: styles.clearButton,
            children: /*#__PURE__*/_jsx(Icon, {
              name: "close",
              size: "s",
              color: "neutralDarker"
            })
          }), hasSelectedValue && !disabled && /*#__PURE__*/_jsx(Box, {
            height: 16,
            width: 1,
            backgroundColor: "neutralLighter",
            marginHorizontal: "xs",
            testID: `${testID}-divider`
          }), /*#__PURE__*/_jsx(Icon, {
            name: isBottomSheetVisible ? 'arrow-up' : 'arrow-down',
            size: "xs",
            color: iconColor,
            testID: `${testID}-icon`,
            accessibilityLabel: `${testID}-icon`
          })]
        })]
      })
    }), (helpText || errorText || successText) && /*#__PURE__*/_jsx(InputHelpText, {
      helpText: helpText,
      errorState: errorState,
      errorText: errorText,
      successState: successState,
      successText: successText
    }), /*#__PURE__*/_jsx(SelectBottomSheet, {
      visible: isBottomSheetVisible,
      title: bottomSheetTitle,
      closeButtonLabel: closeButtonLabel,
      closeButtonAction: closeBottomSheet,
      selectButtonLabel: selectButtonLabel,
      selectButtonAction: confirmSelection,
      options: sortedOptions,
      type: multiple ? 'checkbox' : 'radio',
      onSelect: handleBottomSheetSelection,
      selectedOption: isBottomSheetVisible ? bottomSheetSelectionRef.current : selectedValues,
      testID: `${testID}-bottomSheet`,
      maxVisibleItems: maxVisibleItems
    })]
  });
});
const styles = StyleSheet.create({
  clearButton: {
    padding: 2
  }
});
export default Select;
//# sourceMappingURL=Select.js.map