"use strict";

import React, { createContext, useContext, useEffect, useState } from 'react';
import { FlatList, TouchableOpacity } from 'react-native';
import Box from '../Box/Box';
import Text from '../Text/Text';
import Icon from '../Icon/Icon';
import insertObjectBetweenElements from './utils';
import Badge from '../Badge/Badge';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const TabsContext = /*#__PURE__*/createContext(null);
const Tabs = ({
  value: controlledValue,
  defaultValue,
  onValueChange,
  ...rest
}) => {
  const [value, setValue] = useState(defaultValue ?? '');
  useEffect(() => {
    if (controlledValue !== undefined) {
      setValue(controlledValue);
    }
  }, [controlledValue]);
  const handleChange = newValue => {
    if (controlledValue === undefined) {
      setValue(newValue);
    }
    onValueChange(newValue);
  };
  return /*#__PURE__*/_jsx(TabsContext.Provider, {
    value: {
      value,
      onValueChange: handleChange
    },
    children: /*#__PURE__*/_jsx(Box, {
      testID: "tabsBox",
      accessibilityLabel: "tabsBox",
      flex: 1,
      ...rest
    })
  });
};
const List = ({
  children
}) => {
  const data = React.useMemo(() => {
    const childrenArray = React.Children.toArray(children);
    return insertObjectBetweenElements(childrenArray, /*#__PURE__*/_jsx(Box, {
      flex: 1,
      mx: "m",
      width: 1,
      height: 30,
      alignSelf: "center",
      backgroundColor: "neutralLighter"
    }));
  }, [children]);
  return /*#__PURE__*/_jsx(Box, {
    testID: "tabsListBox",
    accessibilityLabel: "tabsListBox",
    children: /*#__PURE__*/_jsx(FlatList, {
      data: data,
      testID: "tabsFlatList",
      accessibilityLabel: "tabsFlatList",
      keyExtractor: (item, index) => index.toString(),
      horizontal: true,
      showsHorizontalScrollIndicator: false,
      renderItem: ({
        item
      }) => item,
      contentContainerStyle: {
        flexGrow: 1,
        justifyContent: 'space-around'
      }
    })
  });
};
const Option = ({
  value,
  title,
  caption = '',
  disabled = false,
  iconName = undefined,
  badgeText = undefined
}) => {
  const context = useContext(TabsContext);
  if (!context) {
    throw new Error('Tabs.Option must be used within Tabs');
  }
  const isSelected = context.value === value;
  const conditionalColor = disabled ? 'neutralLight' : isSelected ? 'primaryKey' : 'neutralDarker';
  const underline = React.useMemo(() => {
    if (!isSelected) {
      return null;
    }
    return /*#__PURE__*/_jsx(Box, {
      position: "absolute",
      bottom: 0,
      alignSelf: "center",
      width: "100%",
      borderBottomWidth: 2,
      borderColor: "primaryKey"
    });
  }, [isSelected]);
  return /*#__PURE__*/_jsxs(TouchableOpacity, {
    disabled: disabled,
    style: {
      justifyContent: 'center'
    },
    onPress: () => !disabled && context.onValueChange(value),
    children: [/*#__PURE__*/_jsxs(Box, {
      testID: "tabOptionBox",
      alignItems: "center",
      justifyContent: "center",
      paddingVertical: "xs",
      children: [/*#__PURE__*/_jsxs(Box, {
        flexDirection: "row",
        alignItems: "center",
        children: [iconName ? /*#__PURE__*/_jsx(Box, {
          pr: "3xs",
          testID: "tabIconBox",
          children: /*#__PURE__*/_jsx(Icon, {
            name: iconName,
            size: "s",
            color: conditionalColor,
            testID: "tabIcon"
          })
        }) : null, /*#__PURE__*/_jsx(Box, {
          testID: "tabTitleBox",
          children: /*#__PURE__*/_jsx(Text, {
            testID: "tabTitle",
            fontWeight: "500",
            color: conditionalColor,
            numberOfLines: 1,
            ellipsizeMode: "tail",
            children: title
          })
        }), badgeText ? /*#__PURE__*/_jsx(Box, {
          pl: "3xs",
          testID: "tabBadgeBox",
          children: /*#__PURE__*/_jsx(Badge, {
            testID: "tabBadge",
            text: badgeText,
            size: "small",
            transparent: true,
            color: "neutralFull",
            backgroundColor: "dangerKey"
          })
        }) : null]
      }), caption ? /*#__PURE__*/_jsx(Text, {
        testID: "tabCaption",
        fontSize: 12,
        color: disabled ? 'neutralLight' : 'neutralDarker',
        children: caption
      }) : null]
    }), underline]
  });
};
const Content = ({
  value,
  ...rest
}) => {
  const context = useContext(TabsContext);
  if (!context) {
    throw new Error('Tabs.Content must be used within Tabs');
  }
  return context.value === value ? /*#__PURE__*/_jsx(Box, {
    testID: "tabsContentBox",
    accessibilityLabel: "tabsContentBox",
    ...rest
  }) : null;
};
Tabs.List = List;
Tabs.Option = Option;
Tabs.Content = Content;
export default Tabs;
//# sourceMappingURL=Tabs.js.map