"use strict";

import React, { forwardRef, useEffect } from 'react';
import { Easing, TextInput, I18nManager } from 'react-native';
import theme from '../../theme';
import Box from '../Box/Box';
import { TextAreaLabel } from './TextAreaLabel';
import { TextAreaHelpText } from './TextAreaHelpText';
import { TextAreaCounterText } from './TextAreaCounterText';
import { useInputRef, useInputValue, useOutlineLabelVisibility } from './hooks';
import { getBorderColor, getPlaceholderText } from './utils';
import { backgroundColor, border, createRestyleComponent, createVariant, layout, position, spacing } from '@ergenekonyigit/restyle';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const isRTL = I18nManager.isRTL;
const sizeVariant = createVariant({
  property: 'size',
  themeKey: 'textAreaSizeVariants'
});
const BaseTextArea = createRestyleComponent([layout, spacing, border, backgroundColor, position, sizeVariant], TextInput);
const TextArea = /*#__PURE__*/forwardRef(({
  size = 'medium',
  label,
  labelFixed = false,
  placeholder,
  helpText,
  counterText,
  errorText,
  error = false,
  disabled = false,
  editable = true,
  easing = Easing.inOut(Easing.ease),
  testID = 'textArea',
  maxLength = 200,
  maxLengthErrorText = 'You have exceeded the character limit.',
  ...rest
}, ref) => {
  const [focused, setFocused] = React.useState(false);
  const [errorState, setErrorState] = React.useState(false);
  const [length, setLength] = React.useState(0);
  const innerRef = useInputRef();
  const {
    value,
    isControlled,
    setUncontrolledValue
  } = useInputValue({
    value: rest.value,
    defaultValue: rest.defaultValue
  });
  const placeholderText = getPlaceholderText({
    label,
    labelFixed,
    placeholder,
    value,
    focused
  });
  const borderColor = getBorderColor({
    focused,
    errorState
  });
  const textAreaHeight = {
    small: theme.textAreaSizeVariants.small.height,
    medium: theme.textAreaSizeVariants.medium.height,
    large: theme.textAreaSizeVariants.large.height
  }[size];
  const {
    startAnimation,
    stopAnimation,
    animatedViewProps,
    animatedTextProps
  } = useOutlineLabelVisibility({
    theme,
    easing,
    textAreaHeight,
    focused,
    value,
    disabled,
    helpText,
    counterText,
    errorText,
    errorState
  });
  useEffect(() => {
    setErrorState(error);
  }, [error]);
  const handleFocus = async e => {
    if (disabled || !editable) {
      return;
    }
    await startAnimation();
    setFocused(true);
    setErrorState(false);
    rest.onFocus?.(e);
  };
  const handleBlur = async e => {
    if (disabled || !editable) {
      return;
    }
    if (!value) {
      await stopAnimation();
    }
    setFocused(false);
    rest.onBlur?.(e);
  };
  const handleChangeText = nextValue => {
    if (disabled || !editable) {
      return;
    }
    if (!isControlled) {
      setUncontrolledValue(nextValue);
    }
    setLength(nextValue.length);
    rest.onChangeText?.(nextValue);
  };
  React.useImperativeHandle(ref, () => ({
    focus: () => innerRef.current?.focus(),
    clear: () => innerRef.current?.clear(),
    setNativeProps: args => innerRef.current?.setNativeProps(args),
    isFocused: () => innerRef.current?.isFocused() || false,
    blur: () => innerRef.current?.blur()
  }));
  const placeHolderSize = () => {
    if (size === 'large') {
      return 'm';
    } else if (size === 'medium') {
      return 'xs';
    } else {
      return '2xs';
    }
  };
  return /*#__PURE__*/_jsxs(Box, {
    py: "2xs",
    px: "m",
    children: [/*#__PURE__*/_jsx(TextAreaLabel, {
      label: label,
      labelFixed: labelFixed,
      errorState: errorState,
      animatedViewProps: animatedViewProps,
      animatedTextProps: animatedTextProps,
      textAreaHeight: textAreaHeight
    }), /*#__PURE__*/_jsx(Box, {
      flexDirection: "row",
      alignItems: "center",
      borderWidth: 1,
      borderRadius: "m",
      borderColor: length > maxLength ? 'dangerKey' : borderColor,
      backgroundColor: disabled ? 'neutralLightest' : 'neutralFull',
      px: "m",
      justifyContent: "flex-start",
      zIndex: "layer_0",
      accessibilityLabel: `${testID}-box`,
      testID: `${testID}-box`,
      children: /*#__PURE__*/_jsx(BaseTextArea, {
        ...rest,
        flex: 1,
        paddingTop: placeHolderSize(),
        ref: innerRef,
        multiline: true,
        height: textAreaHeight,
        bg: "transparent",
        onFocus: handleFocus,
        onBlur: handleBlur,
        placeholder: placeholderText,
        placeholderTextColor: theme.colors.neutralLight,
        onChangeText: handleChangeText,
        value: value,
        disabled: disabled,
        editable: !disabled,
        accessibilityLabel: testID,
        testID: testID,
        textAlign: isRTL ? 'right' : 'left'
      })
    }), /*#__PURE__*/_jsxs(Box, {
      flexDirection: "row",
      justifyContent: "space-between",
      children: [/*#__PURE__*/_jsx(Box, {
        justifyContent: "flex-start",
        children: /*#__PURE__*/_jsx(TextAreaHelpText, {
          helpText: helpText,
          errorState: length > maxLength ? true : errorState,
          errorText: length > maxLength ? maxLengthErrorText : errorText
        })
      }), /*#__PURE__*/_jsx(Box, {
        justifyContent: "flex-end",
        children: /*#__PURE__*/_jsx(TextAreaCounterText, {
          counterText: length,
          errorState: length > maxLength ? true : errorState
        })
      })]
    })]
  });
});
export default TextArea;
//# sourceMappingURL=TextArea.js.map