"use strict";

import React from 'react';
import { Animated, PanResponder, TouchableOpacity, useWindowDimensions } from 'react-native';
import Text from '../Text/Text';
import { ToastDurations } from './types';
import { capitalizeFirstLetter, getFirstNChar, pausableTimer } from './utils';
import Box from '../Box/Box';
import { createRestyleComponent, createVariant } from '@ergenekonyigit/restyle';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const variantVariant = createVariant({
  property: 'variant',
  themeKey: 'toastVariants'
});
const ToastViewContainer = createRestyleComponent([variantVariant], Animated.View);
const ToastView = /*#__PURE__*/React.forwardRef(({
  variant = 'default',
  text,
  duration = ToastDurations.SHORT,
  onDisappared,
  action,
  actionText,
  ...rest
}, ref) => {
  const dims = useWindowDimensions();
  const anim = React.useRef(new Animated.Value(dims.height / 2)).current;
  const layoutSizesRef = React.useRef();
  const timerRef = React.useRef(pausableTimer(() => {
    playHideAnim();
  }, duration));
  const panResponderRef = React.useRef(PanResponder.create({
    onStartShouldSetPanResponderCapture: (e, gestureState) => {
      return !action || !(gestureState.dx === 0 && gestureState.dy === 0);
    },
    onMoveShouldSetPanResponder: () => true,
    onPanResponderGrant: () => {
      timerRef.current.pause();
    },
    onPanResponderMove: (e, gestureState) => {
      if (gestureState.dy >= 0) {
        Animated.event([null, {
          dy: anim
        }], {
          useNativeDriver: false
        })(e, gestureState);
      }
    },
    onPanResponderRelease: (e, gestureState) => {
      if (gestureState.dy > (layoutSizesRef.current?.height ?? 0) || gestureState.vy > 0.05) {
        playHideAnim();
      } else {
        playShowAnim();
        timerRef.current.resume(2000);
      }
    }
  }));
  React.useImperativeHandle(ref, () => ({
    show() {
      playShowAnim();
    },
    hide() {
      playHideAnim();
    }
  }));
  const playShowAnim = React.useCallback(() => {
    Animated.timing(anim, {
      toValue: 0,
      duration: 333,
      useNativeDriver: true
    }).start();
  }, [anim]);
  const playHideAnim = React.useCallback(() => {
    Animated.timing(anim, {
      toValue: dims.height / 2,
      duration: 333,
      useNativeDriver: true
    }).start(onDisappared);
  }, [anim, dims.height, onDisappared]);
  React.useEffect(() => {
    playShowAnim();
    return () => {
      timerRef.current.kill();
    };
  }, [playShowAnim, timerRef]);
  const onLayout = React.useCallback(event => {
    layoutSizesRef.current = event.nativeEvent.layout;
  }, []);
  const actionView = React.useMemo(() => {
    if (!action) {
      return null;
    }
    return /*#__PURE__*/_jsx(TouchableOpacity, {
      onPress: () => action(playHideAnim),
      testID: "toast-action",
      accessibilityLabel: "toast-action",
      children: /*#__PURE__*/_jsx(Text, {
        variant: "subtitle03Medium",
        color: "neutralDarker",
        testID: "toast-action-text",
        accessibilityLabel: "toast-action-text",
        p: "2xs",
        children: capitalizeFirstLetter(actionText)
      })
    });
  }, [action, actionText, playHideAnim]);
  return /*#__PURE__*/_jsxs(ToastViewContainer, {
    ...panResponderRef.current.panHandlers,
    variant: variant,
    onLayout: onLayout,
    testID: "toast-container",
    accessibilityLabel: "toast-container",
    style: {
      transform: [{
        translateY: anim
      }]
    },
    ...rest,
    children: [/*#__PURE__*/_jsx(Box, {
      flex: 1,
      shadowRadius: 30,
      children: /*#__PURE__*/_jsx(Text, {
        variant: "bodyText",
        color: "neutralDarker",
        testID: "toast-text",
        accessibilityLabel: "toast-text",
        textAlign: "left",
        children: getFirstNChar(text, 250, '...')
      })
    }), actionView]
  });
});
export default ToastView;
//# sourceMappingURL=ToastView.js.map