"use strict";

// @ts-ignore
import { toHaveStyle } from '@testing-library/jest-native';
import React from 'react';
import { cleanup, fireEvent, mockPanResponder, render } from '../../test-utils';
import ToastView from './ToastView';
import { jsx as _jsx } from "react/jsx-runtime";
describe('ToastView', () => {
  expect.extend({
    toHaveStyle
  });
  beforeEach(() => {
    cleanup();
    jest.clearAllMocks();
    jest.clearAllTimers();
  });
  test('should render toast correctly', () => {
    // given
    const data = {
      text: 'lorem ipsum dolor sit amet.'
    };

    // when
    const {
      toJSON
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text
    }));

    // then
    expect(toJSON()).toMatchSnapshot();
  });
  test('should render toast with action correctly', () => {
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: () => null,
      actionText: 'Tamam'
    };

    // when
    const {
      toJSON
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));

    // then
    expect(toJSON()).toMatchSnapshot();
  });
  test('should render given text correctly', () => {
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.'
    };

    // when
    const {
      getByText,
      queryByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text
    }));
    const text = getByText('lorem ipsum dolor sit amet.');
    const actionBtn = queryByTestId('toast-action');

    // then
    expect(actionBtn).toBeFalsy();
    expect(text).toBeTruthy();
  });
  test('should render given text and action text correctly', () => {
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByText,
      queryByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const text = getByText('lorem ipsum dolor sit amet.');
    const actionText = getByText('Tamam');
    const actionBtn = queryByTestId('toast-action');

    // then
    expect(actionBtn).toBeTruthy();
    expect(text).toBeTruthy();
    expect(actionText).toBeTruthy();
  });
  test('should render the default toast with color #FEF2E8', () => {
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastContainer = getByTestId('toast-container');
    // then
    expect(toastContainer).toHaveStyle({
      backgroundColor: '#FEF2E8'
    });
  });
  test('should render the success toast with color #E7F9EF', () => {
    // given
    const data = {
      variant: 'success',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastContainer = getByTestId('toast-container');
    // then
    expect(toastContainer).toHaveStyle({
      backgroundColor: '#E7F9EF'
    });
  });
  test('should render the error toast with color #FFEEEC', () => {
    // given
    const data = {
      variant: 'error',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastContainer = getByTestId('toast-container');
    // then
    expect(toastContainer).toHaveStyle({
      backgroundColor: '#FFEEEC'
    });
  });
  test('should render the warning toast with color #FFF8E6', () => {
    // given
    const data = {
      variant: 'warning',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastContainer = getByTestId('toast-container');
    // then
    expect(toastContainer).toHaveStyle({
      backgroundColor: '#FFF8E6'
    });
  });
  test('should handle the action button press event', () => {
    const mockAction = jest.fn();
    // given
    const data = {
      variant: 'error',
      text: 'lorem ipsum dolor sit amet.',
      action: mockAction,
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastAction = getByTestId('toast-action');
    fireEvent.press(toastAction);
    // then
    expect(mockAction).toBeCalled();
  });
  test('should handle onLayout', () => {
    const mockAction = jest.fn();
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: mockAction,
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const toastContainer = getByTestId('toast-container');
    fireEvent(toastContainer, 'layout', {
      nativeEvent: {
        layout: {
          height: 200
        }
      }
    });
  });
  test('should play hide anim after 7 seconds', () => {
    jest.useFakeTimers();
    const mockAction = jest.fn();
    const mockOnDisappared = jest.fn();
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: mockAction,
      actionText: 'Tamam'
    };

    // when
    render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      duration: data.duration,
      action: data.action,
      actionText: data.actionText,
      onDisappared: mockOnDisappared
    }));
    jest.runAllTimers();
    expect(mockOnDisappared).toBeCalled();
  });
  test('should handle ref show method', () => {
    const toastRef = /*#__PURE__*/React.createRef();
    const mockAction = jest.fn();
    const mockOnDisappared = jest.fn();
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: mockAction,
      actionText: 'Tamam'
    };

    // when
    render(/*#__PURE__*/_jsx(ToastView, {
      ref: toastRef,
      variant: data?.variant,
      text: data?.text,
      duration: data.duration,
      action: data.action,
      actionText: data.actionText,
      onDisappared: mockOnDisappared
    }));
    toastRef.current?.show();
  });
  test('should handle ref hide method', () => {
    const toastRef = /*#__PURE__*/React.createRef();
    const mockAction = jest.fn();
    const mockOnDisappared = jest.fn();
    // given
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: mockAction,
      actionText: 'Tamam'
    };

    // when
    render(/*#__PURE__*/_jsx(ToastView, {
      ref: toastRef,
      variant: data?.variant,
      text: data?.text,
      duration: data.duration,
      action: data.action,
      actionText: data.actionText,
      onDisappared: mockOnDisappared
    }));
    toastRef.current?.hide();
  });
  test('should handle onStartShouldSetPanResponderCapture', async () => {
    // given
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      action: data.action,
      actionText: data.actionText
    }));
    const panHandler = getByTestId('toast-container');
    panHandler?.props.onStartShouldSetResponderCapture({}, {
      dx: 0,
      dy: 0
    });
  });
  test('should handle onResponderGrant', async () => {
    // given
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text
    }));
    const panHandler = getByTestId('toast-container');
    panHandler?.props.onResponderGrant();
  });
  test('should handle onResponderMove', async () => {
    // given
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text
    }));
    const panHandler = getByTestId('toast-container');
    panHandler?.props.onResponderMove({}, {
      dy: 10
    });
  });
  test('should handle onResponderRelease swipe bigger than height', async () => {
    jest.useFakeTimers();
    // given
    const mockOnDisappared = jest.fn();
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      onDisappared: mockOnDisappared
    }));
    const toastContainer = getByTestId('toast-container');
    fireEvent(toastContainer, 'layout', {
      nativeEvent: {
        layout: {
          height: 100
        }
      }
    });
    toastContainer?.props.onResponderRelease({}, {
      dy: 250
    });
    jest.advanceTimersByTime(350);
    expect(mockOnDisappared).toBeCalled();
  });
  test('should handle onResponderRelease swipe lower than height', async () => {
    jest.useFakeTimers();
    // given
    const mockOnDisappared = jest.fn();
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      onDisappared: mockOnDisappared
    }));
    const toastContainer = getByTestId('toast-container');
    fireEvent(toastContainer, 'layout', {
      nativeEvent: {
        layout: {
          height: 100
        }
      }
    });
    toastContainer?.props.onResponderRelease({}, {
      dy: 90
    });
    jest.advanceTimersByTime(350);
    expect(mockOnDisappared).not.toBeCalled();
  });
  test('should handle onResponderRelease high velocity', async () => {
    jest.useFakeTimers();
    // given
    const mockOnDisappared = jest.fn();
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      onDisappared: mockOnDisappared
    }));
    const toastContainer = getByTestId('toast-container');
    fireEvent(toastContainer, 'layout', {
      nativeEvent: {
        layout: {
          height: 100
        }
      }
    });
    toastContainer?.props.onResponderRelease({}, {
      dy: 10,
      vy: 0.051
    });
    jest.advanceTimersByTime(350);
    expect(mockOnDisappared).toBeCalled();
  });
  test('should handle onResponderRelease low velocity', async () => {
    jest.useFakeTimers();
    // given
    const mockOnDisappared = jest.fn();
    mockPanResponder();
    const data = {
      variant: 'default',
      text: 'lorem ipsum dolor sit amet.',
      action: jest.fn(),
      actionText: 'Tamam'
    };

    // when
    const {
      getByTestId
    } = render(/*#__PURE__*/_jsx(ToastView, {
      variant: data?.variant,
      text: data?.text,
      onDisappared: mockOnDisappared
    }));
    const toastContainer = getByTestId('toast-container');
    fireEvent(toastContainer, 'layout', {
      nativeEvent: {
        layout: {
          height: 100
        }
      }
    });
    toastContainer?.props.onResponderRelease({}, {
      dy: 10,
      vy: 0.049
    });
    jest.advanceTimersByTime(350);
    expect(mockOnDisappared).not.toBeCalled();
  });
});
//# sourceMappingURL=ToastView.test.js.map