"use strict";

import React from 'react';
import { SafeAreaProvider } from 'react-native-safe-area-context';
import { cleanup, fireEvent, render, waitFor } from '../../test-utils';
import { deviceHeight } from '../../utils/dimentions';
import Box from '../Box/Box';
import Button from '../Button/Button';
import Text from '../Text/Text';
import { Overlay } from './Overlay';
import Tooltip from './Tooltip';
import { TooltipContext, TooltipProvider, useTooltipContext } from './TooltipContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
describe('Tooltip', () => {
  const mockOverlayMeasure = jest.fn();
  const mockChildWrapperTopMeasure = jest.fn();
  const mockChildWrapperBottomMeasure = jest.fn();
  const mockChildWrapperOnLayoutParam = {
    nativeEvent: {
      layout: {
        width: 200,
        height: 30
      }
    }
  };
  beforeEach(() => {
    mockChildWrapperTopMeasure.mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 200, 30, 16, 16);
    });
    mockChildWrapperBottomMeasure.mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 200, 30, 16, 800);
    });
    mockOverlayMeasure.mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 400, 800, 0, 62);
    });
  });
  afterEach(() => {
    jest.resetAllMocks();
    jest.clearAllMocks();
    cleanup();
  });
  test('should render top correctly', async () => {
    // when
    const {
      toJSON
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    expect(toJSON()).toMatchSnapshot();
  });
  test('should render bottom correctly', async () => {
    // when
    const {
      toJSON
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          position: 'bottom',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    expect(toJSON()).toMatchSnapshot();
  });
  test('should not render without provider', async () => {
    try {
      render(/*#__PURE__*/_jsx(SafeAreaProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      }));
    } catch (error) {
      expect(error.message).toBe('useTooltipContext must be used within a TooltipProvider');
    }
  });
  test('should show top and hide tooltip via trigger', async () => {
    // when
    const mockOnClose = jest.fn();
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    expect(content).not.toBeNull();
    const close = queryByTestId('tooltip-close-test');
    fireEvent.press(close);
    expect(queryByTestId('tooltip-content-test')).toBeNull();
    expect(mockOnClose).toHaveBeenCalledWith('test');
  });
  test('should show bottom and hide tooltip via trigger', async () => {
    // when
    const mockOnClose = jest.fn();
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          overlay: true,
          hole: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'bottom',
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    expect(content).not.toBeNull();
    const close = queryByTestId('tooltip-close-test');
    fireEvent.press(close);
    expect(queryByTestId('tooltip-content-test')).toBeNull();
    expect(mockOnClose).toHaveBeenCalledWith('test');
  });
  test('should show bottom when there is no space at top', async () => {
    // when
    const mockOnClose = jest.fn();
    const {
      queryByTestId,
      getByTestId,
      getByA11yHint
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          overlay: true,
          hole: true,
          position: 'top',
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    await waitFor(() => getByTestId('tooltip-content-test'));
    const popup = queryByTestId('tooltip-test');
    popup.parent.instance.measure = jest.fn().mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 360, 200, 32, 62);
    });
    fireEvent(popup, 'layout', {});
    expect(getByA11yHint('bottom')).not.toBeNull();
  });
  test('should show top when there is no space at bottom', async () => {
    // when
    const mockOnClose = jest.fn();
    const {
      queryByTestId,
      getByTestId,
      getByA11yHint
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          content: 'content Lorem ipsum dolor sit amet',
          overlay: true,
          hole: true,
          position: 'bottom',
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperBottomMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    await waitFor(() => getByTestId('tooltip-content-test'));
    const popup = queryByTestId('tooltip-test');
    popup.parent.instance.measure = jest.fn().mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 360, 100, 32, deviceHeight - 100 - 21); // component height, inset bottom
    });
    fireEvent(popup, 'layout', {});
    expect(getByA11yHint('top')).not.toBeNull();
  });
  test('should render overlay and hole', async () => {
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          overlay: true,
          hole: true,
          position: undefined,
          content: 'content Lorem ipsum dolor sit amet',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    const overlay = queryByTestId('tooltip-overlay-test');
    const hole = queryByTestId('tooltip-hole-test');
    const highlight = queryByTestId('tooltip-highlight-test');
    expect(content).not.toBeNull();
    expect(overlay).not.toBeNull();
    expect(hole).not.toBeNull();
    expect(highlight).toBeNull();
  });
  test('should render overlay and hole and highlight', async () => {
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          overlay: true,
          hole: true,
          highlight: true,
          position: "bottom",
          content: 'content Lorem ipsum dolor sit amet',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const trigger = queryByTestId('tooltip-trigger-test');
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    const overlay = queryByTestId('tooltip-overlay-test');
    const hole = queryByTestId('tooltip-hole-test');
    const highlight = queryByTestId('tooltip-highlight-test');
    expect(content).not.toBeNull();
    expect(overlay).not.toBeNull();
    expect(hole).not.toBeNull();
    expect(highlight).not.toBeNull();
  });
  test('should show and hide via ref', async () => {
    // given
    const ref = /*#__PURE__*/React.createRef();
    const mockOnClose = jest.fn();
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          ref: ref,
          overlay: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    ref.current?.show();
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    expect(content).not.toBeNull();
    ref.current?.hide();
    expect(queryByTestId('tooltip-content-test')).toBeNull();
    expect(mockOnClose).toHaveBeenCalledWith('test');
  });
  test('should clear all tooltips', async () => {
    const ref1 = /*#__PURE__*/React.createRef();
    const ref2 = /*#__PURE__*/React.createRef();
    const Comp = () => {
      const {
        clearAll
      } = useTooltipContext();
      return /*#__PURE__*/_jsxs(Box, {
        children: [/*#__PURE__*/_jsx(Tooltip, {
          id: 'test1',
          ref: ref1,
          overlay: true,
          hole: true,
          content: 'content 1',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text 1"
          })
        }), /*#__PURE__*/_jsx(Tooltip, {
          id: 'test2',
          ref: ref2,
          overlay: true,
          hole: true,
          content: 'content 2',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text 2"
          })
        }), /*#__PURE__*/_jsx(Button, {
          testID: "clear_all_btn",
          onPress: clearAll,
          label: "Clear All"
        })]
      });
    };
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Comp, {})
      })
    }));

    // then
    const childWrapper1 = queryByTestId('tooltip-child-wrapper-test1');
    const childWrapper2 = queryByTestId('tooltip-child-wrapper-test2');
    childWrapper1.parent.instance.measure = mockChildWrapperTopMeasure;
    childWrapper2.parent.instance.measure = jest.fn().mockImplementation(f => {
      // x,y,w,h,px,py
      f(0, 0, 200, 30, 16, 64);
    });
    fireEvent(childWrapper1, 'layout', mockChildWrapperOnLayoutParam);
    fireEvent(childWrapper2, 'layout', mockChildWrapperOnLayoutParam);
    ref1.current?.show();
    ref2.current?.show();
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test1');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content1 = queryByTestId('tooltip-content-test1');
    const content2 = queryByTestId('tooltip-content-test2');
    expect(content1).not.toBeNull();
    expect(content2).toBeNull();

    // clear all
    const btn = queryByTestId('clear_all_btn');
    fireEvent.press(btn);
    expect(queryByTestId('tooltip-content-test1')).toBeNull();
    expect(queryByTestId('tooltip-content-test2')).toBeNull();
  });
  test('should wait for layout ready', async () => {
    const ref = /*#__PURE__*/React.createRef();
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          ref: ref,
          overlay: true,
          hole: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    ref.current?.show();
    expect(queryByTestId('tooltip-content-test')).toBeNull();
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    const content = queryByTestId('tooltip-content-test');
    expect(content).not.toBeNull();
  });
  test('should hide when press overlay', async () => {
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          overlay: true,
          hole: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = mockChildWrapperTopMeasure;
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    const trigger = queryByTestId('tooltip-trigger-test');
    fireEvent.press(trigger);
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    overlayLayout.parent.instance.measure = mockOverlayMeasure;
    fireEvent(overlayLayout, 'layout', {});
    fireEvent.press(overlayLayout);
    expect(queryByTestId('tooltip-content-test')).toBeNull();
  });
  test('should not show when error', async () => {
    // given
    const ref = /*#__PURE__*/React.createRef();
    const mockOnClose = jest.fn();
    const mockOnError = jest.fn();
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          ref: ref,
          overlay: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          highlight: true,
          onClose: mockOnClose,
          onError: mockOnError,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = jest.fn().mockImplementation(f => {
      // x,y,w,h,px,py
      f(undefined, undefined, undefined, undefined, undefined, undefined);
    });
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    ref.current?.show();
    const content = queryByTestId('tooltip-content-test');
    expect(content).toBeNull();
    expect(mockOnClose).not.toBeCalled();
    expect(mockOnError).toHaveBeenCalledWith('test');
  });
  test('should not show when error without onError', async () => {
    // given
    const ref = /*#__PURE__*/React.createRef();
    const mockOnClose = jest.fn();
    // when
    const {
      queryByTestId
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipProvider, {
        children: /*#__PURE__*/_jsx(Tooltip, {
          id: 'test',
          ref: ref,
          overlay: true,
          content: 'content Lorem ipsum dolor sit amet',
          position: 'top',
          highlight: true,
          onClose: mockOnClose,
          children: /*#__PURE__*/_jsx(Text, {
            children: "trigger text"
          })
        })
      })
    }));

    // then
    const childWrapper = queryByTestId('tooltip-child-wrapper-test');
    childWrapper.parent.instance.measure = jest.fn().mockImplementation(f => {
      // x,y,w,h,px,py
      f(undefined, undefined, undefined, undefined, undefined, undefined);
    });
    fireEvent(childWrapper, 'layout', mockChildWrapperOnLayoutParam);
    ref.current?.show();
    const content = queryByTestId('tooltip-content-test');
    expect(content).toBeNull();
    expect(mockOnClose).not.toBeCalled();
  });
  test('should not show overlay when no activeTooltip', async () => {
    // given
    const mockShow = jest.fn();
    const mockHide = jest.fn();
    const mockClearAll = jest.fn();
    const mockSetOverlayViewport = jest.fn();
    // when
    const {
      queryByTestId,
      toJSON
    } = render(/*#__PURE__*/_jsx(SafeAreaProvider, {
      children: /*#__PURE__*/_jsx(TooltipContext.Provider, {
        value: {
          activeTooltip: undefined,
          hide: mockHide,
          show: mockShow,
          clearAll: mockClearAll,
          overlayViewport: {
            x: 0,
            y: 0,
            w: 400,
            h: 800,
            px: 0,
            py: 62
          },
          setOverlayViewport: mockSetOverlayViewport
        },
        children: /*#__PURE__*/_jsx(Overlay, {})
      })
    }));

    // then
    const overlayLayout = queryByTestId('tooltip-overlay-wrapper-test');
    expect(overlayLayout).toBeNull();
    expect(toJSON()).toBeNull();
  });
});
//# sourceMappingURL=Tooltip.test.js.map